#ifndef LIBWCCL_VARIABLES_H
#define LIBWCCL_VARIABLES_H

#include <libwccl/value.h>

namespace Wccl {

/**
 * A class for holding values by name.
 */
class Variables
{
public:
	/// Ctor.
	Variables();

	/**
	 * Set the variable with the given name to the given value.
	 *
	 * If the variable does not exist, it is created.
	 * If it already exists, the value is replaced.
	 */
	void put(const std::string& name, Value* value);

	/**
	 * Get the value of the variable with the given name, or, if it does not
	 * exist, NULL.
	 */
	Value* get_or_null(const std::string& name);

	/**
	 * Get the value of the variable with the given name, or, if it does not
	 * exist, NULL. Const.
	 */
	const Value* get_or_null(const std::string &name) const;

	/**
	 * Get the value of the variable with th egiven name, or, if it does not
	 * exist, create a new empty variable of the type given as the template
	 * parameter and return that.
	 */
	template<typename T>
	Value* get_or_put(const std::string& name) {
		map_t::iterator i = map_.find(name);
		if (i != map_.end()) {
			return i->second;
		} else {
			return map_.insert(std::make_pair(name, new T()))->second;
		}
	}

	/// Typdef for the map type
	typedef std::map<std::string, Value*> map_t;

	/**
	 * Const map accesor, for iterating through all variables
	 */
	const map_t& all_variables() const {
		return map_;
	}

private:
	/// The variables map
	map_t map_;
};

} /* end ns Wccl */

#endif // LIBWCCL_VARIABLES_H
